#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define SKIPRETURN  /* treat carriage retrns in input text as spaces */
#define MAXDEPTH 100    /* hard maximum order to set array size */
char previous[MAXDEPTH+1];
int empty = 0;

#define MAXSIZE 1000000
char book[MAXSIZE+1];

void readbook(char *name)
{
    FILE *fd;
    int booklength;
    int lastspace = 0;
    if (!(fd = fopen(name, "r")))
    {
        perror(name);
        exit(1);
    }
    for (booklength = 0; booklength < MAXSIZE;)
    {
        int c = fgetc(fd);
        if (c <= 0)
            break;
#ifdef SKIPRETURN
        if (c == '\n')
            c = ' ';
#endif
        if (lastspace && (c == ' '))
            continue;
        book[booklength] = c;
        lastspace = (c == ' ');
        booklength++;
    }
    book[booklength] = 0;
    fclose(fd);
}

/* routine to select the next character of output.  "minorder" is the minimum
order desired.  If the order may be increased without dropping below
"mincontinuations" choices, the order is so increased (so to get exactly
"minorder" set mincontinuation to 1000; or to get the maximum order allowed
without dropping below mincontinuation, set minorder to 0.  */

int garbage(int minorder, int mincontinuation)
{
    int ncontinuation[MAXDEPTH], i, j, k, nuse, indx, ret = -1;
    int bign = strlen(book);
    static int ncall = 0;
    
    if (minorder > MAXDEPTH)
        minorder = MAXDEPTH;
    nuse = (minorder >= ncall ? ncall : minorder);
    ncall++;
    if (empty)
        for (i = 0; i < MAXDEPTH; i++)
            previous[i] = 0;
    for (i = 0; i < MAXDEPTH; i++)
        ncontinuation[i] = 0;
    for (i = 0; i < bign; i++)
    {
        for (j = i, k = 0; j >= 0 && k < MAXDEPTH; j--, k++)
        {
            ncontinuation[k]++;
            if (previous[k] != book[j])
                break;
        }
    }
#if 0    /* for debugging - print out analysis of continuations */
    for (i = 0; i < 10; i++)
        printf("%7.4d", ncontinuation[i]);
    printf("\n");
#endif

        /* figure out how many characters to use */
    for (; nuse < MAXDEPTH-1; nuse++)
        if (ncontinuation[nuse+1] < mincontinuation)
            break;
    indx = (random() * (double)(ncontinuation[nuse])) / RAND_MAX;
    if (indx >= ncontinuation[nuse])
        indx--;

    for (i = 0; i < MAXDEPTH; i++)
        ncontinuation[i] = 0;
    /* printf("nuse %d, indx %d prev %s\n", nuse, indx, previous); */
    /* printf("nuse %d\n", nuse); */
    for (i = 0; i < bign; i++)
    {
        for (j = i, k = 0; j >= 0 && k < MAXDEPTH; j--, k++)
        {
            if ((k == nuse) && (ncontinuation[k] == indx))
            {
                int nexti = i+1, nextc, k2;
                if (nexti >= bign)
                    nextc = ' ';
                else nextc = book[nexti];
                printf("%c", nextc);
                fflush(stdout);
                ret = nextc;
                for (k2 = MAXDEPTH-1; k2 >0; k2--)
                    previous[k2] = previous[k2-1];
                previous[0] = nextc;
            }
            ncontinuation[k]++;
            if (previous[k] != book[j])
                break;
        }
    }
    return (ret);
}

main()
{
    int i, last = 0;
    readbook("input.txt");
#if 1
    for (i = 0; i < 10000; i++)
	if (garbage(8, 2) < 0)
            break;
#else
    for (i = 0; i < 10000; i++)
    {
        float nbranch;
        int foo = (random() * 100.) / RAND_MAX;
        nbranch = (last == ' ' ? (foo < 20 ? 2:1)  : 1);
        last = garbage(1, nbranch);
        if (last < 0)
            break;
    }
#endif
    fprintf(stderr, "\n");
}
